﻿--===================================================
--									
--										GHClass
--									ghclass.lua
--	
-- 						(c)2011 The Gryphonheart Team
--								All rights reservated
--===================================================



function GHClass(_className,...)
	local obj,className;
	local parent = _parent;
	obj = CreateFrame("Frame"); -- making it a frame might create more memory use?
	
	local interfaces = {...}
	className = _className;
		
	obj.GetType = function(self)
		return className;
	end
	
	obj.IsClass = function(self,str)
		assert(type(str)=="string","Useage: IsClass(str)");
		return self:GetType() == str;
	end;
	
	obj.IsSameClass = function(self,other)
		assert(type(other)=="table","IsSameClass. Other class is not a table");
		if other.GetType then
			return self:GetType() == other:GetType()
		end
	end
	
	obj.InheritsInterface = function(self,_interface)
		for _,int in pairs(interfaces) do
			if (int == _interface) then
				return true;
			end
		end	
		return false;
	end
	
	obj.Dispose = function(self) -- Standard dispose func. This can be overwritten
		wipe(self);
	end
	
	return obj;
end

function GHCheck(name,expected,acual)
	if not(type(expected)=="table" and type(acual)=="table") then
		error("GHCheck used incorrectly in code. Useage: GHCheck(table,table) got "..type(expected).." "..type(acual),2)
	end
	
	local passed = true;
	local acualRes = {};
	for i,expType in pairs(expected) do
		expType = expType:lower();
		local acualType = type(acual[i]):lower();
		if not(expType == "any") and not(string.find(expType,acualType)) then
			passed = false;
		end		
		acualRes[i] = acualType;
	end
	
	if not(passed) then
		error(name.." Usage: "..string.join(",",unpack(expected)) .. " got "..string.join(" ",unpack(acualRes)),2)
	end
end

function strsubutf8(str,a,b)  -- modified from http://wowprogramming.com/snippets/UTF-8_aware_stringsub_7
	assert(type(str)=="string" and type(a)=="number","incorrect input strsubutf8");
	assert(not(b) or (type(b)=="number" and b <= strlenutf8(str)),"end pos larger than string lenght",b,strlenutf8(str));
	
	b = (b or strlenutf8(str));
	
	
	local start,_end = #str+1,#str+1;
  local currentIndex = 1
  local numChars = 0;
  if a <= 1 then
  	start = a;
  end
  if b <= 1 then
  	_end = b;
  end
  
  while currentIndex <= #str do
    local char = string.byte(str, currentIndex)
    if char > 240 then
      currentIndex = currentIndex + 4
    elseif char > 225 then
      currentIndex = currentIndex + 3
    elseif char > 192 then
      currentIndex = currentIndex + 2
    else
      currentIndex = currentIndex + 1
    end
    
    numChars = numChars +1;
    
    if numChars == a-1 then
    	start = currentIndex;
    end
    if numChars == b then
    	_end = currentIndex-1;
    end
  end
  return str:sub(start, _end)
end